
/* A Configuration represents an arrangement of pieces within the box. */

typedef struct Configuration
{
	struct PieceList	*pieces;						/* the pieces in the configuration */
	int				usesGoal;						/* is the goal piece among them? */
	
	int				distance;						/* these 2 fields are used for BFS, with the configuration treated as */
	struct Configuration	*predecessor;					/* a vertex in the graph of reachable configurations.  See CLRS, p. 532. */

	int				movedPiece;						/* this is additional predecessor information - how we got here from the */
	struct DirectionList	*movedShifts;					/* predecessor configuration.  Used by PrintSolution. */
} Configuration;


Configuration 		*NewConfiguration();
Configuration 		*CopyConfiguration(Configuration *config);
void				DisposeConfiguration(Configuration *config);

void				PuzzleSearch(Configuration *config, int x, int y);
void				Solve(Configuration *config);
struct ConfigurationList	*FindMoves(Configuration *config, int pieceNum);
int					Solved(Configuration *config);

void				PrintSolution(Configuration *config);
void				PrintSolution2(Configuration *config);
void				PrettyPrint(Configuration *config);

int					PlacePiece(Configuration *config, int shapeIndex, int x, int y);
void				RemovePiece(Configuration *config);
int					ConfigOccupies(Configuration *config, int x, int y);

int					SameConfigAs(Configuration *config1, Configuration *config2);


/* A ConfigurationList is a linked list of configuration pointers.  It also has the
   ability to search for a given configuration. */

typedef struct ConfigurationList
{
	Configuration		*config;
	struct ConfigurationList 	*next;
} ConfigurationList;


ConfigurationList	*NewConfigurationList(Configuration *c, ConfigurationList *cl);
void				DeleteCLContents(ConfigurationList *cl);

int					IsInCL(ConfigurationList *cl, Configuration *match);


/* A DirectionList is a linked list of directions.
   Used to record piece shift sequences that make up a move. */

typedef struct DirectionList
{
	int 			dir;
	struct DirectionList	*next;
} DirectionList;


DirectionList	*NewDirectionList(int d, DirectionList *dl);
DirectionList	*CopyDirectionList(DirectionList *dl);

void			DeleteDLContents(DirectionList *dl);

void			PrintDirectionList(DirectionList *dl);
