#include <stdio.h>
#include "inputspec.h"
#include "configuration.h"


/*
 SIMPLE SLIDING-PIECE PUZZLE FINDER

 We are given a box shape, a set of legal piece shapes, a unique goal piece
 shape, and a goal position.  Our job is to find a configuration made up of
 legal piece shapes, which uses the goal piece shape exactly once, such that
 it's as hard as possible to move the goal piece to the goal position.

 This program is just intended to get you started.  It will perform
 correctly, but not particularly efficiently.  See the handout for tips on
 how to improve it.


 Here is the strategy this program uses:

 1. Read specification from input.  See contest handout for format
		definition.
 
 2. Search through all possible puzzle layouts.  Try to fill each vacant spot
		in the box in turn, either with some input piece shape, or with empty
		space.  When the entire box is full, we have a candidate puzzle.
		Search the layout for solutions.

 3. Solution search: Do a BFS, starting from the layout configuration,
		searching for configurations that have the goal piece in the goal
		position.  Stop when we find a solution.

		There are a couple of subtleties here.  The vertices in our graph
		correspond to puzzle configurations that are reachable from the
		starting configuration.  But we don't have these all in a nice list;
		they are implicitly defined.  We generate new configurations on the
		fly, by trying all possible moves from each configuration we reach.
		In order to keep the BFS from running forever, we have to detect when
		we reach a configuration we've seen before.  So we store all
		configurations we've seen during this search in an auxiliary list.
		Before adding a new configuration to the BFS queue, we search this
		list (and the queue) to make sure we haven't already seen it.

		The second subtlety lies in the definition of a move.  Any sequence of
		unit shifts of the same piece counts as a move.  We use another BFS
		to find all available moves of a piece from a given configuration.

 4. When we find a solution, we know the minimum number of moves necessary
		to reach it, by virtue of BFS.  Print out the puzzle and its solution
		if it's longer than the previous longest solution, and terminate the
		solution search.

 5. Keep looping through layouts, until we finish or (more likely) run out
		of time!
 
*/


/* GLOBALS */

InputSpec 	Spec;				/* The input specifications */

int 		MaxDistance;		/* Largest minimum solution length */
 

int main()
{
Configuration *empty;

	ReadSpec();

	empty = NewConfiguration();
	PuzzleSearch(empty, 0, 0);
		
	printf("\nFinished search.  Best solution length: %d.\n", MaxDistance);

	return 0;
}



