#ifndef	PIECE_SEEN
#define PIECE_SEEN

/* PieceShapes hold the 2-d bit-vectors defining shapes of pieces.  If shape
	is a PieceShape, and shape[x][y] is true, then that piece shape contains a
	1x1 square at location (x, y).  (0, 0) is the top left of the piece. */

#define MAXPIECEDIM		6				/* pieces can be at most 6x6 */

typedef int PieceShape[MAXPIECEDIM][MAXPIECEDIM];

struct Configuration;

/* A Piece represents an instance of a piece, with a location. */

typedef struct Piece
{
	int 	top, left;			/* where is the upper left corner of this piece? */
	int		shapeIndex;			/* what shape is it?  (index into shapes array in
									input spec) */
} Piece;

Piece 	*NewPiece(int whichShape, int l, int t);
Piece	*CopyPiece(Piece *piece);

int		PieceSolved(Piece *piece);
int		IsGoalPiece(Piece *piece);

int		PieceOccupies(Piece *piece, int x, int y);
int		CanSlide(Piece *piece, int dir, struct Configuration *config);
void	Shift(Piece *piece, int dir);
	
void 	PrintPiece(Piece *piece);
	
int		SamePieceAs(Piece *piece1, Piece *piece2);


/* A PieceList is a linked list of Piece pointers. */

typedef struct PieceList
{
	Piece				*piece;
	struct PieceList 	*next;
} PieceList;

PieceList 	*NewPieceList(Piece *p, PieceList *pl);
PieceList 	*CopyPieceList(PieceList *pl);

void		DeletePLContents(PieceList *pl);

void		PrintPieceList(PieceList *pl);

Piece		*PLIndex(PieceList *pl, int pieceNum);


#endif
